#! /usr/bin/perl
# original name: AgentVersions.pl
# Purpose: to collect SiteMinder Web Agent API version and IP Address
#
# Changelog:
# JPerlmutter - 2012/05/18
#       * Revised to use SMMON data
# JPerlmutter - 2011/12/13
#	* added CA Standard disclaimer
# JPerlmutter - 2011/04/02
#	* added hash table to count the amount of times an agent is seen
#	* set print out to be from the hash to have one line per agent
# JPerlmutter - 2011/03/29
#	* created script
#

#  NEITHER CA, INC. NOR ANY OF ITS SUBSIDIARIES AND AFFILIATES ("CA") NOR
#  ANY COMMUNITY MEMBER SHALL BE LIABLE TO THE OTHER OR ANY OTHER MEMBER OR THIRD
#  PARTY FOR DIRECT, CONSEQUENTIAL, INCIDENTAL, INDIRECT AND/OR SPECIAL DAMAGES
#  FOR ANY CLAIMS ARISING FROM OR IN ANY WAY CONNECTED WITH YOUR DECISION TO
#  ACCESS OR USE ANY SUCH FILES, EVEN IF THE POSSIBILITY OF SUCH DAMAGES IS, OR
#  SHOULD HAVE BEEN, KNOWN.  THESE FILES ARE PROVIDED AS IS WITHOUT ANY WARRANTY
#  OR REPRESENTATION OF ANY KIND EXPRESS OR IMPLIED INCLUDING WITHOUT LIMITATION
#  ANY IMPLIED WARRANTY OF MERCHANTABILITY/SATISFACTORY QUALITY, FITNESS FOR A
#  PARTICULAR PURPOSE OR NON-INFRINGMENT. Your usage of any such Files is at your
#  own risk.  You are solely responsible for testing such Files prior to
#  implementing them in either a test or production environment.  We encourage you
#  to check for any documentation (if provided) by looking in the document
#  comments or on the message boards for a corresponding thread for additional
#  information (if available). It is recommended to deploy/implement in a test or
#  QA environment before implementing in a production environment. Such Files are
#  not covered by CA's Support Policy and Terms.  CA will not under any
#  circumstances support them.

# modules used
use strict;
use warnings;

# variables used
my $infile; # file to be passed in
my $outfile; # file to print results to
my $Message=9999; # field used to tell where message is in the incoming file
my $Data=9999; # field used to tell where the Agent API is in the incoming file
my $line; # used to analyze the current line
my @input; # used to read in lines
my %seen; # hash for seen agent combinations
my $agent; # key name for agent hash
my $agree; # agreement input for EULA/Disclaimer

# Actual Program

# user interaction
if ($#ARGV == 0){
    if ($ARGV[0] eq "/?"){
	print("Description:\n");
	print("\tThis Program is for finding Agent Versions connecting to a Policy Server\n");
	print("\tIt takes a \"Profiler\" that uses the following minimal configuration:\n");
	print("\t\tcomponents: IsAuthorized/Receive_Request\n");
	print("\t\tdata: AgentName, Message, Data\n");
	print("\t\n");
	print("\tUse without arguments as it prompts for what it needs.\n");
	print("\tIt will remind what the file requirements are before prompting\n");
  print("\t\n");
	exit;
    }
}else{

# get EULA/DISCLAIMER agreement
	print("\tNEITHER CA, INC. NOR ANY OF ITS SUBSIDIARIES AND AFFILIATES (\"CA\") NOR\n");
	print("ANY COMMUNITY MEMBER SHALL BE LIABLE TO THE OTHER OR ANY OTHER MEMBER OR THIRD\n");
	print("PARTY FOR DIRECT, CONSEQUENTIAL, INCIDENTAL, INDIRECT AND/OR SPECIAL DAMAGES\n");
	print("FOR ANY CLAIMS ARISING FROM OR IN ANY WAY CONNECTED WITH YOUR DECISION TO\n");
	print("ACCESS OR USE ANY SUCH FILES, EVEN IF THE POSSIBILITY OF SUCH DAMAGES IS, OR\n");
	print("SHOULD HAVE BEEN, KNOWN.  THESE FILES ARE PROVIDED AS IS WITHOUT ANY WARRANTY\n");
	print("OR REPRESENTATION OF ANY KIND EXPRESS OR IMPLIED INCLUDING WITHOUT LIMITATION\n");
	print("ANY IMPLIED WARRANTY OF MERCHANTABILITY/SATISFACTORY QUALITY, FITNESS FOR A\n");
	print("PARTICULAR PURPOSE OR NON-INFRINGMENT. Your usage of any such Files is at your\n");
	print("own risk.  You are solely responsible for testing such Files prior to\n");
	print("implementing them in either a test or production environment.  We encourage you\n");
	print("to check for any documentation (if provided) by looking in the document\n");
	print("comments or on the message boards for a corresponding thread for additional\n");
	print("information (if available). It is recommended to deploy/implement in a test or\n");
	print("QA environment before implementing in a production environment. Such Files are\n");
	print("not covered by CA\'s Support Policy and Terms.  CA will not under any\n");
	print(" circumstances support them.\n\n");
	print("Please enter \"Yes\" to agree to the Disclaimer/EULA and accept all\n");
	print("\tresponsibility for side effects of running this script.\n");
	$agree=<STDIN>;
	chomp($agree);
	unless($agree eq "Yes") {exit;};

	print("Description:\n");
	print("\tThis Program is for finding Agent Versions connecting to a Policy Server\n");
	print("\tIt takes a \"Profiler\" that uses the following minimal configuration:\n");
	print("\t\tcomponents: IsAuthorized/Receive_Request\n");
	print("\t\tdata: Message, Data\n");
	print("Enter input filename: ");
  $infile=<STDIN>;
  print("Enter output filename: ");
  $outfile=<STDIN>;
  chomp($outfile);
}

# file handle gathering
open(INFILE, "$infile") or die "Could not open $infile for reading.\n";
open(OUTFILE, ">>$outfile") or die "Could not open $outfile for writing.\n";

# read profiler to verify we know where the Message, Data, AgentName and IPAddress are
$line=<INFILE>; # get the top line
chomp $line; # remove extra white space
$line=~s/\[//g; # remove [
@input = split("\]",$line); # make an array of the headers

# loop to get all the needed headers
for(my $i = 0; $i < scalar(@input); $i++) {

	my $field = $input[$i];

	if($field eq "Message") { $Message = $i; }
	if($field eq "Data") { $Data = $i; }
}

# verify we found the needed headers; exit with respective error if not found.
if($Message == 9999){ print STDERR "No Message Field Found. Exiting...\n\n"; exit 1; }
if($Data == 9999){ print STDERR "No Agent Version Field Found. Exiting...\n\n"; exit 2; }

# create a header for this running
my $rightnow = localtime();
print OUTFILE "\n\tCA SiteMinder Web Agent Detection Tool\n";
print OUTFILE "\tThis iteration run: $rightnow.\n\n";

# write the information to a file, nicely formatted
while($line = <INFILE>) { # cycle through getting the the information
	
	chomp $line; # remove extra white space
	$line=~s/\[//g; # remove [
	@input = split("\]",$line); # make an array of the headers
	
	if($input[$Message]=~/Receive request attribute 145.*/i){ # we have an agent connecting
	  $input[$Data]=~/.*?HostId='(.*?)'.*?Name>(.*?)<.*?Platform=(.*?),.*?,FileVersion=(.*?),.*/;
	  $seen{"Found Agent $2 reporting in from $1 on $3 of version $4"}++;
	}
}

foreach $agent (keys %seen){ # for each agent type/name/ip combo seen
	print OUTFILE "$agent $seen{$agent} time(s)\n"; # let us know the agent and how many times it did an isProtected
}

# end program
close(INFILE);
close(OUTFILE);
exit;
